@@ -23,7 +23,6 @@ FRK <- function(f,                     # formula (compulsory)
                 average_in_BAU = TRUE, # average data into BAUs
                 sum_variables = NULL,  # variables to sum rather than average
                 normalise_wts = TRUE,
-                normalise_basis = TRUE,# normalise basis functions
                 fs_model = "ind",      # fine-scale variation component
                 vgm_model = NULL,      # variogram model for error estimation
                 K_type = c("block-exponential", "precision", "unstructured"), # type of K matrix
@@ -37,7 +36,6 @@ FRK <- function(f,                     # formula (compulsory)
                 link = c("identity", "log", "sqrt", "logit", "probit", "cloglog", "inverse", "inverse-squared"),
                 optimiser = nlminb,    # Optimiser for fitting (applicable only if method = 'TMB')
                 fs_by_spatial_BAU = FALSE,
-                include_fs = TRUE,
                 known_sigma2fs = NULL, 
                 taper = NULL, 
                 simple_kriging_fixed = TRUE,
@@ -86,7 +84,6 @@ FRK <- function(f,                     # formula (compulsory)
     .check_args2(n_EM = n_EM, tol = tol, method = method, print_lik = print_lik, 
                  response = response, link = link, K_type = K_type, lambda = lambda,
                  optimiser = optimiser, fs_by_spatial_BAU = fs_by_spatial_BAU, 
-                 include_fs = include_fs,
                  known_sigma2fs = known_sigma2fs, BAUs = BAUs, taper = taper, 
                  simple_kriging_fixed = simple_kriging_fixed, ...)                      
 
@@ -117,7 +114,7 @@ FRK <- function(f,                     # formula (compulsory)
     ## Now construct the BAUs around this dataset
     if(is.null(BAUs)) {
 
-        if(opts_FRK$get("verbose")) cat("Constructing BAUs...\n")
+        cat("Constructing BAUs...\n")
         BAUs <- auto_BAUs(manifold = manifold, # Construct BAUs
                           data = data[[d]],    # Using the dataset with largest extent
                           ...)
@@ -132,7 +129,7 @@ FRK <- function(f,                     # formula (compulsory)
     }
 
     if(is.null(basis)) {
-        if(opts_FRK$get("verbose")) cat("Generating basis functions...\n")
+        cat("Generating basis functions...\n")
         tot_data <- sum(sapply(data,length))         # Total number of data points available
         if(K_type == "unstructured") {               # If unstructured then limit the
             max_sp_basis <- min(tot_data^(0.5),2000) # amount of basis functions to be sqrt
@@ -212,11 +209,9 @@ FRK <- function(f,                     # formula (compulsory)
              fs_model = fs_model,              # fs model (only "ind" for now)
              vgm_model = vgm_model,            # vgm model for error estimation
              K_type = K_type,                  # "block-exponential", "unstructured", "precision"
-             normalise_basis = normalise_basis,
              response = response, 
              link = link, 
-             fs_by_spatial_BAU = fs_by_spatial_BAU, 
-             include_fs = include_fs)                  
+             fs_by_spatial_BAU = fs_by_spatial_BAU)                  
 
     ## After constructing SRE model, fit it
     cat("Fitting SRE model...\n")
@@ -253,7 +248,7 @@ FRK <- function(f,                     # formula (compulsory)
     if(!(is.null(BAUs))) {
         if(!(is(BAUs,"SpatialPolygonsDataFrame") | is(BAUs,"SpatialPixelsDataFrame") | is(BAUs,"STFDF")))
             stop("BAUs should be a SpatialPolygonsDataFrame, SpatialPixelsDataFrame, or a STFDF object")
-        if(!all(sapply(data,function(x) identical(proj4string(x), proj4string(BAUs)))))
+        if(!all(sapply(data,function(x) identical(.rawproj4string(x), .rawproj4string(BAUs)))))
             stop("Please ensure all data items and BAUs have the same coordinate reference system")
         if(!(all(BAUs$fs >= 0)))
             stop("fine-scale variation basis function needs to be nonnegative everywhere")
