@@ -90,7 +90,7 @@ Type objective_function<Type>::operator() ()
   
   // Variance components relating to the basis-function coefficients
   PARAMETER_VECTOR(logsigma2);  vector<Type> sigma2 = exp(logsigma2);
-  PARAMETER_VECTOR(logtau);     // The transformation we apply to tau depends on which K_type is used
+  PARAMETER_VECTOR(logtau);     vector<Type> tau    = exp(logtau);
   PARAMETER(logsigma2_t);       Type sigma2_t       = exp(logsigma2_t);
   PARAMETER(frho_t);            Type rho_t          = transform_minus_one_to_one(frho_t);
   PARAMETER_VECTOR(logdelta);   vector<Type> delta  = exp(logdelta); // only for K_type == "precision-block-exponential"
@@ -159,129 +159,84 @@ Type objective_function<Type>::operator() ()
   
   for (int k = 0; k < nres; k++) { // For each resolution of spatial basis functions
     
-    // Separable is very different, so treat it as a special case
-    if (K_type == "separable") {
-      
-      // NB: "separable" has only been implemented for spatial
-      
-      // Variance components associated with row and column direction
-      // The first half of the sigma2 vector corresponds to row direction variances
-      // and second half the column variances. Same for rho.
-      vector<Type> sigma2_r = sigma2.head(nres);
-      vector<Type> sigma2_c = sigma2.tail(nres);
-      // The correlation parameter in an AR1 must be between -1 and 1. 
-      vector<Type> tau      = transform_minus_one_to_one(logtau);
-      vector<Type> rho_r    = tau.head(nres); 
-      vector<Type> rho_c    = tau.tail(nres);
-      
-      // First construct M_r and M_c
-      Eigen::SparseMatrix<Type> M_r = choleskyAR1(sigma2_r[k], rho_r[k], n_r[k]);
-      Eigen::SparseMatrix<Type> M_c = choleskyAR1(sigma2_c[k], rho_c[k], n_c[k]);
-      
-      // Log-determinant
-      logdetQ_inv += -2.0 * n_c[k] * diagLogSum(M_r);
-      logdetQ_inv += -2.0 * n_r[k] * diagLogSum(M_c);
-      
-      
-      // Quadratic form
-      if (temporal) {
-        // First compute L_{s, k}' H_k
-        // HAVENT IMPLEMENTED SPATIO-TEMPORAL WITH SEPARABLE YET
-        // J.block(start_eta, 0, r_si[k], r_t) = Mk * J.block(start_eta, 0, r_si[k], r_t);
-        
-      } else {
-        matrix<Type> Hi = eta.segment(start_eta, r_si[k]);
-        Hi.resize(n_c[k], n_r[k]);
-        matrix<Type> vi = M_c.transpose() * Hi * M_r;
-        vi.resize(r_si[k], 1); // vec() operator
-        quadform_eta += (vi.array() * vi.array()).sum();
+    // Construct kth block as a sparse matrix: use triplet list (row, column, value)
+    std::vector<T> tripletList;    // Create a vector of triplet lists, called 'tripletList'
+    tripletList.reserve(nnz[k]);   // Reserve number of non-zeros in the matrix
+    
+    // Vector to store the row sums (only applicable for K_type == "precision-block-exponential")
+    vector<Type> rowSums(r_si[k]);
+    rowSums.fill(0);
+    
+    // Compute the matrix coefficients and store them in the triplet list.
+    if (K_type == "neighbour") {
+      for (int j = start_x; j < start_x + nnz[k]; j++) {  
+        (row_indices[j] == col_indices[j]) ? coef = tau[k] * (x[j] + sigma2[k] + epsilon) : coef = -tau[k] * x[j]; 
+        tripletList.push_back(T(row_indices[j] - start_eta, col_indices[j] - start_eta, coef));
       }
+    } else if (K_type == "block-exponential") {
+      for (int j = start_x; j < start_x + nnz[k]; j++) {  
+        coef = (sigma2[k] + epsilon) * exp( -x[j] / (tau[k] + epsilon) ) * pow( 1.0 - x[j] / beta[k], 2.0) * ( 1.0 + x[j] / (2.0 * beta[k]));
+        tripletList.push_back(T(row_indices[j] - start_eta, col_indices[j] - start_eta, coef));
+      }
+    } else if (K_type == "precision-block-exponential") {
       
-    } else {
-      
-      // For all K_type != "separable", we only require tau > 0, rather than -1 < tau < 1.
-      vector<Type> tau = exp(logtau);
-      
-      // Construct kth block as a sparse matrix: use triplet list (row, column, value)
-      std::vector<T> tripletList;    // Create a vector of triplet lists, called 'tripletList'
-      tripletList.reserve(nnz[k]);   // Reserve number of non-zeros in the matrix
-      
-      // Vector to store the row sums (only applicable for K_type == "precision-block-exponential")
-      vector<Type> rowSums(r_si[k]);
-      rowSums.fill(0);
-      
-      // Compute the matrix coefficients and store them in the triplet list.
-      if (K_type == "neighbour") {
-        for (int j = start_x; j < start_x + nnz[k]; j++) {  
-          (row_indices[j] == col_indices[j]) ? coef = tau[k] * (x[j] + sigma2[k] + epsilon) : coef = -tau[k] * x[j]; // x[j] will usually be 1
+      for (int j = start_x; j < start_x + nnz[k]; j++){ 
+        if (col_indices[j] != row_indices[j]) {
+          coef = -sigma2[k] * exp( -x[j] / tau[k] ) * pow(1.0 - x[j] / beta[k], 2.0) * ( 1.0 + x[j] / (2.0 * beta[k]));
           tripletList.push_back(T(row_indices[j] - start_eta, col_indices[j] - start_eta, coef));
-        }
-      } else if (K_type == "block-exponential") {
-        for (int j = start_x; j < start_x + nnz[k]; j++) {  
-          coef = (sigma2[k] + epsilon) * exp( -x[j] / (tau[k] + epsilon) ) * pow( 1.0 - x[j] / beta[k], 2.0) * ( 1.0 + x[j] / (2.0 * beta[k]));
-          tripletList.push_back(T(row_indices[j] - start_eta, col_indices[j] - start_eta, coef));
-        }
-      } else if (K_type == "precision-block-exponential") {
-        
-        for (int j = start_x; j < start_x + nnz[k]; j++){ 
-          if (col_indices[j] != row_indices[j]) {
-            coef = -sigma2[k] * exp( -x[j] / tau[k] ) * pow(1.0 - x[j] / beta[k], 2.0) * ( 1.0 + x[j] / (2.0 * beta[k]));
-            tripletList.push_back(T(row_indices[j] - start_eta, col_indices[j] - start_eta, coef));
-            rowSums[row_indices[j] - start_eta] += coef;
-          }
-        }
-        // Add the diagonal elements (these depend on the row sums)
-        for (int j = 0; j < r_si[k]; j++) {
-          tripletList.push_back(T(j, j, delta[k] - rowSums[j]));
+          rowSums[row_indices[j] - start_eta] += coef;
         }
       }
-      
-      
-      // Convert triplet list of non-zero entries to a true SparseMatrix.
-      // NB: mat is "Kk" the variance matrix if K_type == "block-exponential",
-      // and the precision matrix "Qk" for all other formulations.
-      SpMat mat(r_si[k], r_si[k]);
-      mat.setFromTriplets(tripletList.begin(), tripletList.end());
-      
-      // Compute the (upper) Cholesky factor of mat
-      Eigen::SimplicialLLT< SpMat, Eigen::Upper > llt;
-      llt.compute(mat);
-      SpMat Uk = llt.matrixU();
-      
-      // Log-determinant
-      if (K_type == "block-exponential") {
-        logdetQ_inv += 2.0 * r_t * diagLogSum(Uk);
-      } else {
-        logdetQ_inv += -2.0 * r_t * diagLogSum(Uk);
-      }
-      
-      // P (the permutation matrix)
-      Eigen::PermutationMatrix<Eigen::Dynamic> P = llt.permutationP();
-      
-      // Construct the matrix Mk such that Qk = Mk' Mk.
-      SpMat Mk(r_si[k], r_si[k]);
-      if (K_type != "block-exponential") { // We don't explicitly construct Mk with block-exponential
-        Mk = Uk * P; 
-      } 
-      
-      // Quadratic form
-      if (temporal) {
-        if (K_type == "block-exponential") {
-          J.block(start_eta, 0, r_si[k], r_t) = (Uk.transpose()).template triangularView<Eigen::Lower>().solve(P * J.block(start_eta, 0, r_si[k], r_t));
-        } else {
-          J.block(start_eta, 0, r_si[k], r_t) = Mk * J.block(start_eta, 0, r_si[k], r_t);
-        }
-      } else {
-        vector<Type> vk(r_si[k]);
-        if (K_type == "block-exponential") {
-          vk = ((Uk.transpose()).template triangularView<Eigen::Lower>().solve(P * eta.segment(start_eta, r_si[k]).matrix())).array();
-        } else {
-          vk = Mk * eta.segment(start_eta, r_si[k]).matrix();
-        }
-        quadform_eta += (vk * vk).sum();
+      // Add the diagonal elements (these depend on the row sums)
+      for (int j = 0; j < r_si[k]; j++) {
+        tripletList.push_back(T(j, j, delta[k] - rowSums[j]));
       }
     }
     
+    // Convert triplet list of non-zero entries to a true SparseMatrix.
+    // NB: mat is "Kk" the variance matrix if K_type == "block-exponential",
+    // and the precision matrix "Qk" for all other formulations.
+    SpMat mat(r_si[k], r_si[k]);
+    mat.setFromTriplets(tripletList.begin(), tripletList.end());
+    
+    // Compute the (upper) Cholesky factor of mat
+    Eigen::SimplicialLLT< SpMat, Eigen::Upper > llt;
+    llt.compute(mat);
+    SpMat Uk = llt.matrixU();
+    
+    // Log-determinant
+    if (K_type == "block-exponential") {
+      logdetQ_inv += 2.0 * r_t * diagLogSum(Uk);
+    } else {
+      logdetQ_inv += -2.0 * r_t * diagLogSum(Uk);
+    }
+    
+    // P (the permutation matrix)
+    Eigen::PermutationMatrix<Eigen::Dynamic> P = llt.permutationP();
+    
+    // Construct the matrix Mk such that Qk = Mk' Mk.
+    SpMat Mk(r_si[k], r_si[k]);
+    if (K_type != "block-exponential") { // We don't explicitly construct Mk with block-exponential
+      Mk = Uk * P; 
+    } 
+    
+    // Quadratic form
+    if (temporal) {
+      if (K_type == "block-exponential") {
+        J.block(start_eta, 0, r_si[k], r_t) = (Uk.transpose()).template triangularView<Eigen::Lower>().solve(P * J.block(start_eta, 0, r_si[k], r_t));
+      } else {
+        J.block(start_eta, 0, r_si[k], r_t) = Mk * J.block(start_eta, 0, r_si[k], r_t);
+      }
+    } else {
+      vector<Type> vk(r_si[k]);
+      if (K_type == "block-exponential") {
+        vk = ((Uk.transpose()).template triangularView<Eigen::Lower>().solve(P * eta.segment(start_eta, r_si[k]).matrix())).array();
+      } else {
+        vk = Mk * eta.segment(start_eta, r_si[k]).matrix();
+      }
+      quadform_eta += (vk * vk).sum();
+    }
+      
     start_eta += r_si[k];
     start_x   += nnz[k];
   }
