@@ -19,8 +19,8 @@ test_that("other linalg functions", {
                as.numeric(b  %*% solve(A) %*% t(b)))
   expect_is(Takahashi_Davis(A),"dgCMatrix")
   expect_equal(cholPermute(A),cholPermute(A))
-  #expect_equal(cholsolve(A,b),solve(A) %*% b)
-  #expect_equal(cholsolve(A,b,perm = T),solve(A) %*% b)
-  #expect_equal(cholsolve(A,b,perm = T,cholQp = X$Qpermchol,P = X$P),solve(A) %*% b)
+  expect_equal(cholsolve(A,b),solve(A) %*% b)
+  expect_equal(cholsolve(A,b,perm = T),solve(A) %*% b)
+  expect_equal(cholsolve(A,b,perm = T,cholQp = X$Qpermchol,P = X$P),solve(A) %*% b)
 
 })
